﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

/*
 * AMN 10-22-2018 - This is the initial release of the Qualsoft Windows driver sample application.
 *                  This is intended to work with version 2.0.0.79 and 2.0.2.80 of the Windows driver.
 *                  Any time a new version of the driver is released, the DEVMODE struct may change.
 *                  Changes to the DEVMODE struct will require changes to this application to make 
 *                  them match.
 */

namespace ProgrammaticPrinting
{
    public partial class Form1 : Form
    {
        bool imageLoaded = false;
        Image loadedImage;
        string imagePath = "None";

        const string imageMustBeLoadedStr = "An image must first be selected using the Load Image button.";

        public Form1()
        {
            InitializeComponent();
        }

        private void loadImageButton_Click(object sender, EventArgs e)
        {
            OpenFileDialog loadImageDialog = new OpenFileDialog();
            loadImageDialog.Filter = "Bitmaps (*.bmp)|*.bmp";
            loadImageDialog.DefaultExt = "bmp";
            loadImageDialog.AddExtension = true;
            if (loadImageDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK && loadImageDialog.FileName.EndsWith(".bmp"))
            {
                imagePath = loadImageDialog.FileName;
                try
                {
                    loadedImage = Image.FromFile(imagePath);
                    imageLoaded = true;
                    currentImageLabel.Text = "Current Image: " + imagePath;
                } catch (Exception ex)
                {
                    MessageBox.Show("Loading the image failed.  " + Environment.NewLine + ex.ToString());
                }
            }
            else if (!loadImageDialog.FileName.EndsWith(".bmp"))
                MessageBox.Show("The selected file is not a bitmap.  Confirm that the selected file name ends with \".bmp\".");
        }

        private void printButton_Click(object sender, EventArgs e)
        {
            if (imageLoaded)
                print();
            else
                MessageBox.Show(imageMustBeLoadedStr);
        }

        public void print()
        {
            if (!imageLoaded)
            {
                MessageBox.Show(imageMustBeLoadedStr);
                return;
            }

            //PrintDialog printDialog = new PrintDialog();//These lines can be used to modify the print preferences through the dialog prior to printing.
            //printDialog.ShowDialog();                   //These lines can be used to modify the print preferences through the dialog prior to printing.

            PrintDocument document = new PrintDocument();
            document.PrintPage += printPageEvent;
            document.Print();
        }

        private void printPageEvent(object sender, PrintPageEventArgs e)
        {
            e.Graphics.PageScale = 1/3f;
            if (loadedImage != null)
                e.Graphics.DrawImageUnscaledAndClipped(loadedImage, new Rectangle(0, 0, loadedImage.Width, loadedImage.Height));
        }

        private void modifyPreferencesButton_Click(object sender, EventArgs e)
        {
            if (!imageLoaded)
            {
                MessageBox.Show(imageMustBeLoadedStr);
                return;
            }

            #region get DEVMODE size
            IntPtr lhPrinter = new IntPtr();
            PrintDirect.OpenPrinter("Microcom 485TM (300dpi Cutter)", ref lhPrinter, 0);
            int dmSize = PrintDirect.DocumentProperties(IntPtr.Zero, lhPrinter, null, IntPtr.Zero, IntPtr.Zero, 0);
            if (dmSize < 0)
            {
                Win32Exception we = new Win32Exception(Marshal.GetLastWin32Error());
                throw we;
            }
            #endregion

            #region get DEVMODE structure
            IntPtr outPtr = Marshal.AllocHGlobal(dmSize);
            int dpResult = PrintDirect.DocumentProperties(IntPtr.Zero/*this.Handle*/, lhPrinter, null, outPtr, IntPtr.Zero, PrintDirect.DM_OUT_BUFFER);
            PrintDirect.DEVMODE devmode = new PrintDirect.DEVMODE();
            devmode = (PrintDirect.DEVMODE)Marshal.PtrToStructure(outPtr, devmode.GetType());
            #endregion

            //make changes to devmode

            #region pass DEVMODE structure back in
            IntPtr inPtr = Marshal.AllocHGlobal(dmSize);
            Marshal.StructureToPtr(devmode, inPtr, false);
            dpResult = PrintDirect.DocumentProperties(IntPtr.Zero/*this.Handle*/, lhPrinter, null, IntPtr.Zero, inPtr, PrintDirect.DM_IN_BUFFER);
            Console.WriteLine("DocumentProperties result: " + dpResult);
            devmode = (PrintDirect.DEVMODE)Marshal.PtrToStructure(outPtr, devmode.GetType());
            #endregion

            #region print page
            PrintDocument document = new PrintDocument();
            document.PrinterSettings.SetHdevmode(inPtr);
            document.PrintPage += printPageEvent;
            document.Print();
            #endregion

            PrintDirect.ClosePrinter(lhPrinter);
        }
    }


    public class PrintDirect
    {
        public const int ERROR_INSUFFICIENT_BUFFER = 122;
        public const int DM_UPDATE = 1;
        public const int DM_COPY = 2;
        public const int DM_PROMPT = 4;
        public const int DM_MODIFY = 8;
        public const int DM_IN_BUFFER = DM_MODIFY;
        public const int DM_IN_PROMPT = DM_PROMPT;
        public const int DM_OUT_BUFFER = DM_COPY;
        public const int DM_OUT_DEFAULT = DM_UPDATE;

        public const int MAX_PRINTER_OPTIONS = 64;
        public const int DEVMODE_MAXCART = 8;
        public const int MAXLENBC = 20;
        public const int MAX_EPS_FILE = 40;

        public const long DM_FORMNAME = 0x00010000L;
        public const long DM_PAPERSIZE = 0x00000002L;
        public const long DM_PAPERLENGTH = 0x00000004L;
        public const long DM_PAPERWIDTH = 0x00000008L;
        public const long DM_ORIENTATION = 0x00000001L;

        [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = false, CallingConvention = CallingConvention.StdCall)]
        public static extern long OpenPrinter(String pPrinterName, ref IntPtr phPrinter, int pDefault);

        [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = false, CallingConvention = CallingConvention.StdCall)]
        public static extern long StartDocPrinter(IntPtr hPrinter, int Level, IntPtr pDocInfo);

        [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = false, CallingConvention = CallingConvention.StdCall)]
        public static extern long StartPagePrinter(IntPtr hPrinter);

        [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = false, CallingConvention = CallingConvention.StdCall)]
        public static extern bool WritePrinter(IntPtr hPrinter, string data, int buf, ref int pcWrittern);

        [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = false, CallingConvention = CallingConvention.StdCall)]
        public static extern bool ReadPrinter(IntPtr hPrinter, ref string data, int buf, ref int pcNoBytesRead);

        [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = false, CallingConvention = CallingConvention.StdCall)]
        public static extern long EndPagePrinter(IntPtr hPrinter);

        [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = false, CallingConvention = CallingConvention.StdCall)]
        public static extern long EndDocPrinter(IntPtr hPrinter);

        [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = false, CallingConvention = CallingConvention.StdCall)]
        public static extern long ClosePrinter(IntPtr hPrinter);

        [DllImport("winspool.drv", CharSet = CharSet.Unicode, ExactSpelling = false, CallingConvention = CallingConvention.StdCall)]
        public static extern bool GetPrinter(IntPtr hPrinter, int Level, IntPtr pPrinter, int cbBuf, ref int pcbNeeded);

        [DllImport("winspool.drv", EntryPoint = "SetPrinterA", SetLastError = false, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public static extern bool SetPrinter(IntPtr hPrinter, int Level, IntPtr pPrinter, int Command);

        [DllImport("winspool.Drv", EntryPoint = "DocumentPropertiesW", SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public static extern int DocumentProperties(IntPtr hwnd, IntPtr hPrinter, [MarshalAs(UnmanagedType.LPWStr)] string pDeviceName, IntPtr pDevModeOutput, IntPtr pDevModeInput, int fMode);


        private const short CCDEVICENAME = 32;
        private const short CCFORMNAME = 32;

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        public struct DEVMODE
        {

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = CCDEVICENAME)]
            public string dmDeviceName;
            public short dmSpecVersion;
            public short dmDriverVersion;
            public short dmSize;
            public short dmDriverExtra;
            public int dmFields;


            [StructLayout(LayoutKind.Explicit)]
            struct union
            {
                [StructLayout(LayoutKind.Explicit)]
                struct unionStruct1
                {
                    [FieldOffset(0)]
                    public short dmOrientation;//new
                    [FieldOffset(2)]
                    public short dmPaperSize;//new
                    [FieldOffset(4)]
                    public short dmPaperLength;//new
                    [FieldOffset(6)]
                    public short dmPaperWidth;//new
                    [FieldOffset(8)]
                    public short dmScale;//new
                    [FieldOffset(10)]
                    public short dmCopies;//new
                    [FieldOffset(12)]
                    public short dmDefaultSource;//new
                    [FieldOffset(14)]
                    public short dmPrintQuality;//new
                }
                [StructLayout(LayoutKind.Explicit)]
                struct dmPosition
                {
                    [FieldOffset(0)]
                    public int dmPositionX;
                    [FieldOffset(4)]
                    public int dmPositionY;
                }
                [StructLayout(LayoutKind.Explicit)]
                struct unionStruct2
                {
                    [StructLayout(LayoutKind.Explicit)]
                    struct dmPosition
                    {
                        [FieldOffset(0)]
                        public int dmPositionX;
                        [FieldOffset(4)]
                        public int dmPositionY;
                    }
                    [FieldOffset(8)]
                    public int dmDisplayOrientation;
                    [FieldOffset(12)]
                    public int dmDisplayFixedOutput;
                }
            }



            //unknowns are used to offset the following values to their correct locations
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 4)]
            public int[] unknowns;




            public short dmColor;
            public short dmDuplex;
            public short dmYResolution;
            public short dmTTOption;
            public short dmCollate;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = CCFORMNAME)]
            public string dmFormName;
            public short dmUnusedPadding;//dmLogPixels?
            public int dmBitsPerPel;
            public int dmPelsWidth;
            public int dmPelsHeight;
            public int dmDisplayFlags;

            //public int dmNup;//new

            public int dmDisplayFrequency;

            public int dmICMMethod;//new
            public int dmICMIntent;//new
            public int dmMediaType;//new
            public int dmDitherType;//new
            public int dmReserved1;//new
            public int dmReserved2;//new
            public int dmPanningWidth;//new
            public int dmPanningHeight;//new


            //
            //
            //Above this line is the public devmode
            //Below this line is the private devmode
            //
            //


            //////////////////////////////////////////////////////////////////////////////
            public uint dwPrivDATA;                     // //*Unique 32-bit signature field
            //////////////////////////////////////////////////////////////////////////////	
            public uint dwFlags;                        // flag bits

            public ushort wChecksum;                      // PPD file checksum
            public ushort wOptionCount;                   // number of options to follow

            [MarshalAs(UnmanagedType.ByValArray, SizeConst = MAX_PRINTER_OPTIONS)]
            public byte[] options;   // printer options

            public short nPrintMethod;
            public short nPrintType;
            public short nPrintMedia;
            public short nDarkness;
            public short nAdjustDelay;
            public short nContrast;
            public short nFeedMode;
            public short nCutInterval;
            public short nLTSSensitivity;

            //public byte bAutoStartStop;//changed to bool 10-19-2018 AMN
            [MarshalAs(UnmanagedType.Bool)]
            public bool bAutoStartStop;

            public short nStartAdjust;
            public short nStopAdjust;
            public short nLeftEdge;
            public short nRightEdge;
            public short nTopEdge;
            public short nBottomEdge;
            public short nDot2Gap;
            public short TopAdjustment;
            public short Rotate180;
            public short PrintButton;
            public short AllowBackFeed;
            public short FullFeed;
            public short ELevel;
            public short FontSet;
            public short PrintMode;
            public short ScalingPercent;

            public short prtResFac;                  //*printer resolution shift factor
            public short prtCaps;                    //*bit field of printer capabilites
            public short prtCaps2;                   //*2nd bit field of printer caps
            public short paperInd;                   //*idx in PaperList of supported papers
            public short prtIndex;                   //*index DEVNAME in string table
            public short trayIndex;                  //*index in PaperList of supported trays

            [MarshalAs(UnmanagedType.ByValArray, SizeConst = DEVMODE_MAXCART)]
            public short[] cartIndex; // index in string table for cart
            public short numCartridges;              // number of cartridges selected
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = DEVMODE_MAXCART)]
            public short[] cartind;   // index to cartridge font
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = DEVMODE_MAXCART)]
            public short[] cartcount; // number of cartridge fonts
            public short romind;                     // index from ROM_ESC_BASE to rom fonts
            public short romcount;                   // number of rom fonts
            public short availmem;                   // available mem in Kbytes
            public byte pageprotect;        		// 0=OFF, 1=LTR, 2=LGL
            public byte reartray;           		// 0=no, 1=yes
            public byte offset;             		// 0=no, 1=yes        aaddai
            public short maxPgSoft;                  // limit of soft fonts per page
            public short maxSoft;                    // max softfonts that can be downloaded
            public short txwhite;                    // white text intensity
            public short dlgoptions;                 // bit field for options dialog
            public short fsvers;                     // fontSummary version number

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAXLENBC)]
            public string szFriendly;       //*currently active barcode

            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 20)]
            public uint[] iDeviceRes;             // device resolutions
            public short xdpi;                       //*chosen x res
            public short ydpi;                       //*chosen y res
            public short txtxdpi;                       //*chosen x res
            public short txtydpi;                       //*chosen x res

            public short DefModule;                  //*0=A 1=B 2=C etc.. 
            public short Feature;                    //*Feature selection bit
            public short nLineHeight;                     //*Peeler / Rewinder
            public short nGapHeight;
            public short nRegOffset;
            public short RibbonEnd;                  //*RibbonEnd Sensor
            public short AutoCutter;                 //*AutoCutter
            public short Thermal;                    //*Thermal 
            public short HeadTemp;                   //*Head Temp 
            public short PrintSpeed;                 //*Print Speed 
            public short PaperSpeed;                 //*Paper Speed 
            public short NumCuts;                    //*Number Cuts 
            public short nDot2GapBlackLine;          //*Dot to Gap value for blackline
            public short EOJFeed;                    //*EOJ feed 
            public short FlashJob;                   //*Flash Job 
            public short OffsetX;                    //*Offset X 
            public short OffsetY;                    //*Offset Y 
            public short CutoffY;                    //*Cutoff Y 
            public short Continuous;                 //*Continuous Paper? 
            public short Units;                      //*Units 
            public short EdgeSense;                  //*Edge Sense 
            public short PrePrinted;                  //*PrePrinted Stationary? 
            public short GapLength;                  //*Gap between labels
            
            //public byte SavewithForm;//changed to bool 10-19-2018 AMN
            [MarshalAs(UnmanagedType.Bool)]
            public bool SavewithForm;

            public short Buffer;                     //*Buffer Mode 

            //50    
            public short nDot2GapDefault;            //*Default/normal dot to gap value
            public short BatchMode;                  //*Batch Printing option
            public short PauseLabel;                 //*Pause Printing option
            public short PauseBatch;                 //*Pause Printing option
            public short PauseIdle;                  //*Pause Printing option

            //public byte AutoSPG;//changed to bool 10-19-2018 AMN
            [MarshalAs(UnmanagedType.Bool)]
            public bool AutoSPG;

            public short SPG;                        //*SPG 	
            public short AGD;                        //*ACTIVATE GAP DETECTION AFTER 'x' STEPS
            public short HeadWidthdots;              //*head width in dots 	

            //60    
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
            public string Port;                   //*Port name devmode assigned to
            //124    
            public short IncrDecr;                   //
            public short IncrAmount;                 //

            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 17)]
            public byte[] FlashJobName;           //*Flash Job Name
            public short IncrFrequency;              //

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 2)]
            public string PadChar;                 //       

            public short MaxFieldNo;                 //
            
            
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 128)]
            public byte[] UCmdJobStart;          //*User Commands!
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 128)]
            public byte[] UCmdJobEnd;            //*User Commands!
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 128)]
            public byte[] UCmdLabStart;          //*User Commands!
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 128)]
            public byte[] UCmdLabEnd;            //*User Commands!



            public byte TTRaster;           		// 0=off, 1=on

            public short Resolution;                  //*chosen res from list box.
            public short Gamma;                      //*chosen intensity
            public short TextOnly;                   //*Text only mode
            public short TextOnlyNew;                //*Text only New mode
            public short density;                    //*density setting
            public short SlashZero;                  //*Slash Zero 

            //public byte bDisableStatusPolling;//changed to bool 10-19-2018 AMN
            [MarshalAs(UnmanagedType.Bool)]
            public bool bDisableStatusPolling;  	// stop polling

            public short Printer;                    //*Current Printer Model 
            public short iTotalJobPages;             // populated by pp?
            public short Handle2;                   //*

            public short WithoutHR;                  //*Without HR bars
            public short WithChecksum;               //*With CS bars
            public short DraftBar;                   //*Print Draft Bars
            public short bUsePrinterConfig;          //*Use Printer Defaults
            public short LangNo;                     //*Selected language

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
            public string szDefaultForm;
            public ushort MaxWidth;
            public ushort MaxLength;
            public ushort CustomWidth;
            public ushort CustomLength;

            public short PresentMode;
            public short PresentTimeout;
            public short PresentMaxDistance;
            public short PresentLoopHeight;
            public short PresentPreFeedSpeed;
            public short PresentPresentSpeed;
            public short PresentRetractSpeed;

            public short UseRibbon;                  //*Thermal or Direct?
            public short PitchY;                     //*Paper Y Pitch 
            public short FeedOffset;                 //*
            public short ExtraFeed;                  //* 
            public short BackFeedOffset;             //*
            public short Mirror;                     //*
            public short RMirror;                    //*
            public short HeadWidth;                     //*Paper X 
            public short PaperX;                     //*Paper X 
            public short PaperY;                     //*Paper Y 

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
            public string PaperSelect;            //*Paper Size Selection codes

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
            public string DriverPath;

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
            public string szPrinterName;

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_EPS_FILE)]
            public string wstrEPSFile;      // EPS file name

            public COLORADJUSTMENT coloradj;                   // structure for halftoning
            public HLFTONE halftone;                   //*halftoning
            public PAGEPROT pageprot;					// page protection

            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 5)]
            public ushort[] wBits;					//*special bit fields (see gui3.h)

            public IntPtr hLangLibInst;               //HANDLE to message lib    
            public IntPtr hBmpLibInst;               //HANDLE to message lib  
            //////////////////////////////////////////////////////////////////////////////
            public uint dwPrivDATA2;                // //*Unique 32-bit signature field
            //////////////////////////////////////////////////////////////////////////////	
        }

        public enum HLFTONE {
        	HT_OPTPHOTO = 0,				// 0	brighten := FALSE	grayscale := TRUE
	        HT_OPTSCANNED = 1,				// 1	brighten := TRUE	grayscale := TRUE
	        HT_OPTLINEART = 2				// 2	brighten := FALSE	grayscale := FALSE
        }

        public enum PAGEPROT {
        	PROT_OFF = 0,					// 0
	        PROT_AUTO = 1,					// 1
        	PROT_LETTER = 2,				// 2
        	PROT_A4 = 3,					// 3
        	PROT_LEGAL14 = 4				// 4
        }

        public struct  COLORADJUSTMENT {    /* ca */
            short caSize;
            short caFlags;
            short caIlluminantIndex;
            short caRedGamma;
            short caGreenGamma;
            short caBlueGamma;
            short caReferenceBlack;
            short caReferenceWhite;
            short caContrast;
            short caBrightness;
            short caColorfulness;
            short caRedGreenTint;
        }

    }
}
